#pragma OPENCL EXTENSION cl_khr_fp64 : enable

#define PI (3.1415926535897932384626433832795)
#define EPSILON (0.000000001)

__kernel void pathTrace(__global uint* output,
		int buff_w, int buff_h, int nx, int ny,
		ulong time) {

	uint ix = get_global_id(0);
	uint iy = get_global_id(1);
	uint width;
	uint height;
	uint xoff = ix * (buff_w / nx);
	uint yoff = iy * (buff_h / ny);

	uint fov = 70;
	double fov_tan = tan((70 / 360.) * (PI / 2));

	double dx, dy, dz;
	double ox = 0;
	double oy = 5 + (time % 1000) / 20.;
	double oz = 0;
	double r, sr = 2;

	if (ix < nx-1)
		width = buff_w / nx;
	else
		width = buff_w - (buff_w / nx) * (nx-1);

	if (iy < ny-1)
		height = buff_h / ny;
	else
		height = buff_h - (buff_h / ny) * (ny-1);

	for (int x = 0; x < width; ++x) {
		for (int y = 0; y < height; ++y) {
			uint index = (yoff + y) * buff_w + (xoff + x);

			dx = fov_tan * (-.5 + (double) (y + yoff) / buff_h);
			dy = -1;
			dz = fov_tan * (.5 - (double) (x + xoff) / buff_w);

			r = sqrt(dx*dx + dy*dy + dz*dz);

			dx /= r;
			dy /= r;
			dz /= r;

			double B = 2 * (dx*ox + dy*oy + dz*oz);
			double C = (ox*ox + oy*oy + oz*oz) - sr*sr;
			double radicand = B*B - 4*C;
			if (radicand <= EPSILON) {
				output[index] = 0xFF << 24;
				continue;
			}
			double discSqrt = sqrt(radicand);
			double t0;
			if (B < 0)
				t0 = (-B - discSqrt) / 2;
			else
				t0 = (-B + discSqrt) / 2;

			if (t0 <= EPSILON) {
				output[index] = 0xFF << 24;
				continue;
			}

			int cv = ((oy - t0) / sr) * 255;

			output[index] = (0xFF << 24) | (cv << 16) | (0xFF << 8) | 0xFF;
		}
	}
}
