/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.main;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Collection;

import javax.swing.JDialog;
import javax.swing.JPanel;

import se.llbit.chunky.renderer.RenderController;
import se.llbit.chunky.world.ChunkPosition;

@SuppressWarnings("serial")
public class Chunk3DView extends JDialog {
    
    private Canvas3D canvas;
    
    int x0, y0;

	private RenderControls renderTools;

	private RenderStatusListener renderStatusListener;

    public Chunk3DView(final Chunky chunky,
	    Collection<ChunkPosition> collection, int nRenderThreads) {
        super(chunky.frame);
        
        JPanel panel = new JPanel(new BorderLayout());
        getContentPane().add(panel);
        
        canvas = new Canvas3D();
        panel.add(canvas, BorderLayout.CENTER);
        
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setModalityType(Dialog.ModalityType.MODELESS);
        
        setTitle("3D Chunk View");
        pack();
        setLocationRelativeTo(chunky);
        setVisible(true);
        requestFocus();
        
        canvas.init();
        
        final RenderController renderController = new RenderController(
                chunky.getWorld(), this, canvas, collection, nRenderThreads);
        renderController.start();
        
        try {
			renderController.waitReady();
			renderTools = new RenderControls(chunky, renderController, this);
			Point loc = getLocation();
			renderTools.setLocation(loc.x + getWidth(), loc.y);
	        renderTools.setVisible(true);
	        renderTools.addListeners();
	        renderStatusListener = new RenderStatusListener(renderController, renderTools);
	        renderStatusListener.start();
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		}
        
        canvas.addKeyListener(new KeyListener() {
            @Override
            public void keyTyped(KeyEvent e) {
                onKeyDown(e);
            }
            @Override
            public void keyReleased(KeyEvent e) {
            }
            @Override
            public void keyPressed(KeyEvent e) {
                onKeyDown(e);
            }
            private void onKeyDown(KeyEvent e) {
                switch (e.getKeyCode()) {
                case KeyEvent.VK_A:
                case KeyEvent.VK_LEFT:
                    renderController.strafeLeft(
			chunky.getShiftModifier() ? .1 : 1);
                    break;
                case KeyEvent.VK_D:
                case KeyEvent.VK_RIGHT:
                    renderController.strafeRight(
			chunky.getShiftModifier() ? .1 : 1);
                    break;
                case KeyEvent.VK_W:
                case KeyEvent.VK_UP:
                    renderController.moveForward(
			chunky.getShiftModifier() ? .1 : 1);
                    break;
                case KeyEvent.VK_S:
                case KeyEvent.VK_DOWN:
                    renderController.moveBackward(
			chunky.getShiftModifier() ? .1 : 1);
                    break;
                case KeyEvent.VK_K:
                    renderController.moveForward(100);
                    break;
                case KeyEvent.VK_J:
                    renderController.moveBackward(100);
                    break;
                case KeyEvent.VK_R:
                    renderController.moveUp(
			chunky.getShiftModifier() ? .1 : 1);
                    break;
                case KeyEvent.VK_F:
                    renderController.moveDown(
			chunky.getShiftModifier() ? .1 : 1);
                    break;
                }
            }
        });
        
        canvas.addMouseMotionListener(new MouseMotionListener() {
            @Override
            public void mouseMoved(MouseEvent e) {
            }
            @Override
            public void mouseDragged(MouseEvent e) {
                int dx = e.getX() - x0;
                int dy = e.getY() - y0;
                renderController.rotateView(
                        - (Math.PI / 250) * dx,
                        (Math.PI / 250) * dy);
                x0 = e.getX();
                y0 = e.getY();
            }
        });
        canvas.addMouseListener(new MouseListener() {
            @Override
            public void mouseReleased(MouseEvent e) {
            }
            @Override
            public void mousePressed(MouseEvent e) {
                x0 = e.getX();
                y0 = e.getY();
            }
            @Override
            public void mouseExited(MouseEvent e) {
            }
            @Override
            public void mouseEntered(MouseEvent e) {
            }
            @Override
            public void mouseClicked(MouseEvent e) {
            }
        });
        
        addWindowListener(new WindowListener() {
			@Override
			public void windowOpened(WindowEvent e) {
			}
			@Override
			public void windowIconified(WindowEvent e) {
			}
			@Override
			public void windowDeiconified(WindowEvent e) {
			}
			@Override
			public void windowDeactivated(WindowEvent e) {
			}
			@Override
			public void windowClosing(WindowEvent e) {
				renderController.interrupt();
				renderTools.dispose();
				renderStatusListener.interrupt();
			}
			@Override
			public void windowClosed(WindowEvent e) {
			}
			@Override
			public void windowActivated(WindowEvent e) {
			}
		});
    }
}
