/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.model;

import se.llbit.chunky.world.Texture;
import se.llbit.math.AABB;
import se.llbit.math.Color;
import se.llbit.math.Ray;

/**
 * A sandstone block.
 * @author Jesper Öqvist <jesper@llbit.se>
 */
public class SandstoneModel {
	private static final AABB block = new AABB(0, 1, 0, 1, 0, 1);

	private static final Texture[][] tex = {

		// normal
		{
			Texture.sandstoneSide,
			Texture.sandstoneSide,
			Texture.sandstoneSide,
			Texture.sandstoneSide,
			Texture.sandstoneTop,
			Texture.sandstoneBottom,
		},
		
		// decorated
		{
			Texture.sandstoneDecorated,
			Texture.sandstoneDecorated,
			Texture.sandstoneDecorated,
			Texture.sandstoneDecorated,
			Texture.sandstoneTop,
			Texture.sandstoneTop,
		},
		
		// smooth
		{
			Texture.sandstoneSmooth,
			Texture.sandstoneSmooth,
			Texture.sandstoneSmooth,
			Texture.sandstoneSmooth,
			Texture.sandstoneTop,
			Texture.sandstoneTop,
		},
	};
	
	public static boolean intersect(Ray ray) {
		int kind = ray.getBlockMetadata() % 3;
		ray.t = Double.POSITIVE_INFINITY;
		if (block.intersect(ray)) {
			int textureId;
			if (ray.n.z == 1) {
				textureId = 0;
			} else if (ray.n.z == -1) {
				textureId = 1;
			} else if (ray.n.x == 1) {
				textureId = 2;
			} else if (ray.n.x == -1) {
				textureId = 3;
			} else if (ray.n.y == 1) {
				textureId = 4;
			} else {// if (ray.intersectionNormal.y == -1) {
				textureId = 5;
			}
			int rgb = tex[kind][textureId].getColor(ray.u, ray.v);
			int alpha = rgb >>> 24;
			if (alpha != 0) {
				ray.color.w = alpha / 255.;
				Color.getRGBComponents(rgb, ray.color);
				ray.x.scaleAdd(ray.tNear, ray.d, ray.x);
				return true;
			}
		}
		return false;
	}
}
