/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.model;

import se.llbit.chunky.world.Texture;
import se.llbit.math.Color;
import se.llbit.math.Quad;
import se.llbit.math.Ray;

public class TallGrassModel extends SpriteModel {

	private static final Texture[] tex = {
		Texture.deadBush, Texture.tallGrass, Texture.fern };

	public static boolean intersect(Ray ray) {
		boolean hit = false;
		ray.t = Double.POSITIVE_INFINITY;
		for (Quad quad : quads) {
			if (quad.intersect(ray)) {
				int kind = ray.getBlockMetadata() % 3;
				int rgb = tex[kind].getColor(ray.u, ray.v);
				int alpha = rgb >>> 24;
				if (alpha != 0) {
					ray.color.w = alpha / 255.;
					Color.getRGBComponents(rgb, ray.color);
					if (kind != 0) {
						int biomeColor = ray.getBiomeColor();
						ray.color.x *= (0xFF & (biomeColor >>> 16)) / 255.;
						ray.color.y *= (0xFF & (biomeColor >>> 8)) / 255.;
						ray.color.z *= (0xFF & biomeColor) / 255.;
					}
					ray.n.set(quad.n);
					ray.n.scale(-Math.signum(ray.d.dot(quad.n)));
					ray.t = ray.tNear;
					hit = true;
				}
			}
		}
		if (hit) {
			ray.x.scaleAdd(ray.t, ray.d, ray.x);
		}
		return hit;
	}
}
