/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.model;

import se.llbit.chunky.renderer.RenderController;
import se.llbit.chunky.world.Texture;
import se.llbit.math.AABB;
import se.llbit.math.Color;
import se.llbit.math.Ray;

/**
 * A water block. The height of the top water block is slightly
 * lower than a regular block.
 * @author Jesper Öqvist <jesper@llbit.se>
 */
public class WaterModel {
	private static AABB[] block = {
		new AABB(0, 1, 0, .875, 0, 1),
		new AABB(0, 1, 0, 1, 0, 1)
	};
	private static double opacity = .36;

	public static boolean intersect(Ray ray) {
		ray.t = Double.POSITIVE_INFINITY;
		if (block[(ray.currentMaterial >> 16) & 1].intersect(ray)) {
			int rgb = Texture.water.getColor();
			ray.color.w = opacity;
			Color.getRGBComponents(rgb, ray.color);
			ray.x.scaleAdd(ray.tNear, ray.d, ray.x);
			
			// Water normal displacement
			if (!RenderController.stillWater && ray.n.y > Ray.EPSILON) {
				
				double bx = ray.x.x + .7 * Math.sin(ray.x.z);
				//double bz = 3 * ray.intersection.z;
				double scale = .02;
				double h = scale * Math.sin(bx);
				double dydx = scale * Math.cos(bx);
				double dydz = 0;
				double yv = 1 - h;
				ray.n.set(dydx, yv, dydz);
				ray.n.normalize();
			}
			
			return true;
		}
		return false;
	}
}
