/* Copyright (c) 2010-2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.world;

import java.awt.Color;
import java.awt.Graphics;
import java.io.File;

import se.llbit.chunky.main.RenderBuffer;

public class NullChunk extends Chunk {
	
	@Override
	public boolean isNullChunk() {
		return true;
	}

	public static NullChunk instance = new NullChunk();

	private NullChunk() {
		super(ChunkPosition.get(0, 0));
		surface = Layer.corruptLayer;
		caves = Layer.corruptLayer;
		layer = Layer.corruptLayer;
	}
	
	@Override
    public synchronized void getBlockData(File dataDirectory, byte[] blocks, byte[] data, byte[] biomes) {
	    for (int i = 0; i < X_MAX * Y_MAX * Z_MAX; ++i)
	        blocks[i] = 0;
	    
	    for (int i = 0; i < X_MAX * Z_MAX; ++i)
            biomes[i] = 0;
	    
	    for (int i = 0; i < (X_MAX * Y_MAX * Z_MAX) / 2; ++i)
            data[i] = 0;
    }

    private NullChunk(ChunkPosition pos) {
		super(pos);
	}
	
	@Override
	protected void renderLayer(RenderBuffer rbuff, int cx, int cz) {
		renderEmpty(rbuff, cx, cz);
	}

	@Override
	protected void renderSurface(RenderBuffer rbuff, int cx, int cz) {
		renderEmpty(rbuff, cx, cz);
	}

	@Override
	protected void renderCaves(RenderBuffer rbuff, int cx, int cz) {
		renderEmpty(rbuff, cx, cz);
	}

	private void renderEmpty(RenderBuffer rbuff, int cx, int cz) {
		ChunkView view = rbuff.getView();
		int scale = view.scale / 16;
		int width = rbuff.getWidth();
		int x0 = view.scale * (cx - view.ix0);
		int z0 = view.scale * (cz - view.iz0);
		
		Graphics g = rbuff.getGraphics();
		g.fillRect(width-z0-view.scale, x0, view.scale, view.scale);
		
		int lx0 = width-z0-view.scale;
		int ly0 = x0;
		g.setColor(Color.black);
		g.drawLine(lx0, ly0+8*scale, lx0+8*scale, ly0+view.scale);
		g.drawLine(lx0, ly0, lx0+view.scale, ly0+view.scale);
		g.drawLine(lx0+8*scale, ly0, lx0+view.scale, ly0+8*scale);
	}
}
