/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.world;

import java.awt.image.BufferedImage;

import se.llbit.math.Ray;
import se.llbit.resources.ImageLoader;
import se.llbit.util.ImageTools;

public class Texture {

	public static final Texture air = new Texture("air");
	public static final Texture stone = new Texture("stone");
	public static final Texture dirt = new Texture("dirt");
	public static final Texture grassSideSaturated = new Texture("grass-side-saturated");
	public static final Texture grassTop = new Texture("grass");
	public static final Texture grassSide = new Texture("air");// TODO
	public static final Texture water = new Texture("water");
	public static final Texture cauldronSide = new Texture();
	public static final Texture cauldronInside = new Texture();
	public static final Texture cauldronTop = new Texture();
	public static final Texture cauldronBottom = new Texture();
	public static final Texture slabTop = new Texture("stone-slab");
	public static final Texture slabSide = new Texture("double-stone-slab");
	public static final Texture brick = new Texture("bricks");
	public static final Texture tntTop = new Texture();
	public static final Texture tntSide = new Texture("tnt");
	public static final Texture tntBottom = new Texture();
	public static final Texture cobweb = new Texture("cobweb");
	public static final Texture redRose = new Texture("rose");
	public static final Texture yellowFlower = new Texture("yellow-flower");
	public static final Texture portal = new Texture("nether-portal");
	public static final Texture cobblestone = new Texture("cobblestone");
	public static final Texture bedrock = new Texture("bedrock");
	public static final Texture sand = new Texture("sand");
	public static final Texture gravel = new Texture("gravel");
	public static final Texture woodTop = new Texture("wood-top");
	public static final Texture ironBlock = new Texture("iron-block");
	public static final Texture goldBlock = new Texture("gold-block");
	public static final Texture diamondBlock = new Texture("diamond-block");
	public static final Texture chestTop = new Texture();
	public static final Texture chestSide = new Texture();
	public static final Texture chestFront = new Texture("chest");
	public static final Texture redMushroom = new Texture("red-mushroom");
	public static final Texture brownMushroom = new Texture("brown-mushroom");
	public static final Texture goldOre = new Texture("gold-ore");
	public static final Texture ironOre = new Texture("iron-ore");
	public static final Texture coalOre = new Texture("coal-ore");
	public static final Texture bookshelf = new Texture("bookshelf");
	public static final Texture mossStone = new Texture("moss-stone");
	public static final Texture obsidian = new Texture("obsidian");
	public static final Texture workbenchTop = new Texture();
	public static final Texture workbenchSide = new Texture("workbench");
	public static final Texture workbenchFront = new Texture("workbench");
	public static final Texture furnaceTop = new Texture();
	public static final Texture furnaceSide = new Texture();
	public static final Texture furnaceUnlitFront = new Texture("furnace");
	public static final Texture furnaceLitFront = new Texture("furnace-lit");
	public static final Texture dispenserFront = new Texture("dispenser");
	public static final Texture sponge = new Texture("sponge");
	public static final Texture glass = new Texture("glass");
	public static final Texture glassPaneSide = new Texture();
	public static final Texture diamondOre = new Texture("diamond-ore");
	public static final Texture redstoneOre = new Texture("redstone-ore");
	public static final Texture stoneBrick = new Texture("stone-bricks");
	public static final Texture mossyStoneBrick = new Texture();
	public static final Texture crackedStoneBrick = new Texture();
	public static final Texture circleStoneBrick = new Texture();
	public static final Texture monsterSpawner = new Texture("spawner");
	public static final Texture snowBlock = new Texture("snow");
	public static final Texture snowSide = new Texture("dirt");// TODO
	public static final Texture ice = new Texture("ice");
	public static final Texture cactusTop = new Texture();
	public static final Texture cactusSide = new Texture();
	public static final Texture cactusBottom = new Texture();
	public static final Texture clay = new Texture("clay");
	public static final Texture sugarCane = new Texture("sugar-canes");
	public static final Texture jukeboxSide = new Texture();
	public static final Texture jukeboxTop = new Texture();
	public static final Texture torch = new Texture("torch");
	public static final Texture woodenDoorTop = new Texture();
	public static final Texture woodenDoorBottom = new Texture();
	public static final Texture ironDoorTop = new Texture();
	public static final Texture ironDoorBottom = new Texture();
	public static final Texture ladder = new Texture("ladder");
	public static final Texture trapdoor = new Texture("trapdoor");
	public static final Texture ironBars = new Texture("iron-bars");
	public static final Texture farmlandWet = new Texture("soil");// TODO: wet variant
	public static final Texture farmlandDry = new Texture("soil");// TODO: dry variant
	public static final Texture lever = new Texture();
	public static final Texture redstoneTorchOn = new Texture("redstone-torch-on");
	public static final Texture redstoneTorchOff = new Texture("redstone-torch-off");
	public static final Texture redstoneWireCross = new Texture("redstone-wire-off-intersect");
	public static final Texture redstoneWire = new Texture();
	public static final Texture pumpkinTop = new Texture();
	public static final Texture pumpkinSide = new Texture();
	public static final Texture pumpkinFront = new Texture();
	public static final Texture jackolanternFront = new Texture();
	public static final Texture netherrack = new Texture("netherrack");
	public static final Texture soulsand = new Texture("soul-sand");
	public static final Texture glowstone = new Texture("glowstone");
	public static final Texture sandstoneSide = new Texture("sandstone");
	public static final Texture sandstoneTop = new Texture();
	public static final Texture sandstoneBottom = new Texture();
	public static final Texture sandstoneDecorated = new Texture();
	public static final Texture sandstoneSmooth = new Texture();
	public static final Texture bedFootTop = new Texture();
	public static final Texture bedHeadTop = new Texture();
	public static final Texture bedFootEnd = new Texture();
	public static final Texture bedFootSide = new Texture();
	public static final Texture bedHeadSide = new Texture();
	public static final Texture bedHeadEnd = new Texture();
	public static final Texture deadBush = new Texture("dead-bush");
	public static final Texture tallGrass = new Texture("tall-grass");
	public static final Texture fern = new Texture();
	public static final Texture vines = new Texture("vines");
	public static final Texture wheat1 = new Texture();
	public static final Texture wheat2 = new Texture();
	public static final Texture wheat3 = new Texture();
	public static final Texture wheat4 = new Texture();
	public static final Texture wheat5 = new Texture();
	public static final Texture wheat6 = new Texture();
	public static final Texture wheat7 = new Texture();
	public static final Texture wheat8 = new Texture();
	public static final Texture rails = new Texture("minecart-track");
	public static final Texture railsCurved = new Texture();
	public static final Texture poweredRailsOn = new Texture();
	public static final Texture poweredRailsOff = new Texture();
	public static final Texture detectorRails = new Texture();
	public static final Texture whiteWool = new Texture("wool");// TODO: variants
	public static final Texture orangeWool = new Texture("wool");// TODO: variants
	public static final Texture magentaWool = new Texture("wool");// TODO: variants
	public static final Texture lightBlueWool = new Texture("wool");// TODO: variants
	public static final Texture yellowWool = new Texture("wool");// TODO: variants
	public static final Texture limeWool = new Texture("wool");// TODO: variants
	public static final Texture pinkWool = new Texture("wool");// TODO: variants
	public static final Texture grayWool = new Texture("wool");// TODO: variants
	public static final Texture lightGrayWool = new Texture("wool");// TODO: variants
	public static final Texture cyanWool = new Texture("wool");// TODO: variants
	public static final Texture purpleWool = new Texture("wool");// TODO: variants
	public static final Texture blueWool = new Texture("wool");// TODO: variants
	public static final Texture brownWool = new Texture("wool");// TODO: variants
	public static final Texture greenWool = new Texture("wool");// TODO: variants
	public static final Texture redWool = new Texture("wool");// TODO: variants
	public static final Texture blackWool = new Texture("wool");// TODO: variants
	public static final Texture lava = new Texture("lava");
	public static final Texture lapislazuliOre = new Texture("lapis-lazuli-ore");
	public static final Texture lapislazuliBlock = new Texture("lapis-lazuli-block");
	public static final Texture pistonSide = new Texture("piston");
	public static final Texture pistonFront = new Texture();
	public static final Texture pistonBack = new Texture();
	public static final Texture pistonArm = new Texture("piston-extension");
	public static final Texture stickyPistonArm = new Texture();
	public static final Texture fire = new Texture("fire");
	public static final Texture redstoneRepeaterOn = new Texture("redstone-repeater-off");
	public static final Texture redstoneRepeaterOff = new Texture("redstone-repeater-on");
	public static final Texture redstoneLampOn = new Texture("redstone-lamp-off");
	public static final Texture redstoneLampOff = new Texture("redstone-lamp-on");
	public static final Texture endPortalFrameTop = new Texture();
	public static final Texture endPortalFrameSide = new Texture();
	public static final Texture endPortal = new Texture("end-portal");
	public static final Texture endStone = new Texture("end-stone");
	public static final Texture brewingStandSide = new Texture();
	public static final Texture brewingStandBase = new Texture();
	public static final Texture cakeTop = new Texture();
	public static final Texture cakeSide = new Texture();
	public static final Texture cakeInside = new Texture();
	public static final Texture cakeBottom = new Texture();
	public static final Texture hugeBrownMushroom = new Texture();
	public static final Texture hugeRedMushroom = new Texture();
	public static final Texture mushroomStem = new Texture();
	public static final Texture mushroomInside = new Texture();
	public static final Texture melonSide = new Texture();
	public static final Texture melonTop = new Texture();
	public static final Texture melonStem = new Texture();
	public static final Texture stemWithMelon = new Texture();
	public static final Texture myceliumTop = new Texture();
	public static final Texture myceliumSide = new Texture();
	public static final Texture lilyPad = new Texture("lily-pad");
	public static final Texture netherBrick = new Texture();
	public static final Texture netherWart1 = new Texture();
	public static final Texture netherWart2 = new Texture();
	public static final Texture netherWart3 = new Texture();
	public static final Texture doubleChestFrontLeft = new Texture();
	public static final Texture doubleChestFrontRight = new Texture();
	public static final Texture doubleChestBackLeft = new Texture();
	public static final Texture doubleChestBackRight = new Texture();
	public static final Texture enchantmentTableSide = new Texture();
	public static final Texture enchantmentTableTop = new Texture();
	public static final Texture enchantmentTableBottom = new Texture();
	public static final Texture eyeOfTheEnder = new Texture();
	public static final Texture dragonEgg = new Texture();
	public static final Texture unknown = new Texture("unknown");
	
	// Tree variants
	
	public static final Texture oakLeaves = new Texture("leaves");// TODO: variants
	public static final Texture pineLeaves = new Texture("leaves");// TODO: variants
	public static final Texture birchLeaves = oakLeaves;
	public static final Texture jungleTreeLeaves = new Texture("leaves");// TODO: variants
	
	public static final Texture oakSapling = new Texture("sapling");// TODO: variants
	public static final Texture pineSapling = new Texture("sapling");// TODO: variants
	public static final Texture birchSapling = new Texture("sapling");// TODO: variants
	public static final Texture jungleTreeSapling = new Texture("sapling");// TODO: variants
	
	public static final Texture oakPlanks = new Texture("wooden-planks");// TODO: variants
	public static final Texture pinePlanks = new Texture("wooden-planks");// TODO: variants
	public static final Texture birchPlanks = new Texture("wooden-planks");// TODO: variants
	public static final Texture jungleTreePlanks = new Texture("wooden-planks");// TODO: variants
	
	public static final Texture oakWood = new Texture("wood");// TODO: variants
	public static final Texture pineWood = new Texture("wood");// TODO: variants
	public static final Texture birchWood = new Texture("wood");// TODO: variants
	public static final Texture jungleTreeWood = new Texture("wood");// TODO: variants
	
	public static final Texture[] wood =
		{ oakWood, pineWood, birchWood, jungleTreeWood };

	public static final Texture[] woodPlank =
		{ oakPlanks, pinePlanks, birchPlanks, jungleTreePlanks };
	
	public static final Texture[] leaves =
		{ oakLeaves, pineLeaves, birchLeaves, jungleTreeLeaves };
	
	public static final Texture[] sapling = {
		oakSapling, pineSapling, birchSapling, jungleTreeSapling };
	
	public static final Texture[] wheat =
		{ wheat1, wheat2, wheat3, wheat4, wheat5, wheat6, wheat7, wheat8 };
	
	public static final Texture[] woodenDoor =
		{ woodenDoorBottom, woodenDoorTop };
	
	public static final Texture[] ironDoor =
		{ ironDoorBottom, ironDoorTop };
	
	public static final Texture[] poweredRails =
		{ poweredRailsOff, poweredRailsOn };
	
	public static final Texture[] railsType =
		{ rails, rails, rails, rails, rails, rails,
		railsCurved, railsCurved, railsCurved, railsCurved };
	
	public static final Texture[] wool =
		{ whiteWool, orangeWool, magentaWool, lightBlueWool,
		yellowWool, limeWool, pinkWool, grayWool, lightGrayWool,
		cyanWool, purpleWool, blueWool, brownWool, greenWool,
		redWool, blackWool };
	
	public static final Texture[] stoneBrickType =
		{ stoneBrick, mossyStoneBrick, crackedStoneBrick, circleStoneBrick };
	
	private BufferedImage image;
	private int avgColor;
	
	public Texture() {
		this(ImageLoader.get("missing-image"));
	}
	
	public Texture(String resourceName) {
		setTexture(ImageLoader.get("textures/" + resourceName + ".png"));
	}
	
	public Texture(BufferedImage img) {
		setTexture(img);
	}
	
	public void setTexture(BufferedImage newImg) {
		image = newImg;
		avgColor = ImageTools.calcAvgColor(image);
	}

	public int getColor(double u, double v) {
		return image.getRGB(
				(int) (u * image.getWidth() - Ray.EPSILON),
				(int) ((1-v) * image.getHeight() - Ray.EPSILON));
	}
	
	/**
	 * @return The average color of this texture
	 */
	public int getColor() {
		return avgColor;
	}

	public BufferedImage getImage() {
		return image;
	}
}
