/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.math;

public final class Color {
    
    private Color() { }

    public static final int getRGB(float r, float g, float b) {
        return 0xFF000000 |
            ((int)(255 * r + .5f) << 16) |
            ((int)(255 * g + .5f) << 8) |
            (int)(255 * b + .5f);
    }
    
    public static final int getRGB(double r, double g, double b) {
        return 0xFF000000 |
            ((int)(255 * r + .5) << 16) |
            ((int)(255 * g + .5) << 8) |
            (int)(255 * b + .5);
    }
    
    public static final int getRGBA(float r, float g, float b, float a) {
        return ((int)(255 * a + .5f) << 24) |
            ((int)(255 * r + .5f) << 16) |
            ((int)(255 * g + .5f) << 8) |
            (int)(255 * b + .5f);
    }
    
    public static final int getRGBA(double r, double g, double b, double a) {
        return ((int)(255 * a + .5) << 24) |
            ((int)(255 * r + .5) << 16) |
            ((int)(255 * g + .5) << 8) |
            (int)(255 * b + .5);
    }

    public static final void getRGBComponents(int irgb, float[] frgb) {
        frgb[0] = (0xFF & (irgb >> 16)) / 255.f;
        frgb[1] = (0xFF & (irgb >> 8)) / 255.f;
        frgb[2] = (0xFF & irgb) / 255.f;
    }
    
    public static final void getRGBComponents(int irgb, Vector4d v) {
        v.x = (0xFF & (irgb >> 16)) / 255.f;
        v.y = (0xFF & (irgb >> 8)) / 255.f;
        v.z = (0xFF & irgb) / 255.f;
    }
    
    public static final void getRGBComponents(int irgb, double[] frgb) {
        frgb[0] = (0xFF & (irgb >> 16)) / 255.0;
        frgb[1] = (0xFF & (irgb >> 8)) / 255.0;
        frgb[2] = (0xFF & irgb) / 255.0;
    }
    
    public static final void getRGBAComponents(int irgb, float[] frgb) {
        frgb[3] = (irgb >>> 24) / 255.f;
        frgb[0] = (0xFF & (irgb >> 16)) / 255.f;
        frgb[1] = (0xFF & (irgb >> 8)) / 255.f;
        frgb[2] = (0xFF & irgb) / 255.f;
    }

    public static final void getRGBAComponents(int irgb, Vector4d v) {
        v.w = (irgb >>> 24) / 255.f;
        v.x = (0xFF & (irgb >> 16)) / 255.f;
        v.y = (0xFF & (irgb >> 8)) / 255.f;
        v.z = (0xFF & irgb) / 255.f;
    }

    public static final void getRGBAComponents(int irgb, double[] frgb) {
        frgb[3] = (irgb >>> 24) / 255.0;
        frgb[0] = (0xFF & (irgb >> 16)) / 255.0;
        frgb[1] = (0xFF & (irgb >> 8)) / 255.0;
        frgb[2] = (0xFF & irgb) / 255.0;
    }

    public static int getRGB(float[] frgb) {
        return 0xFF000000 |
            ((int)(255 * frgb[0] + .5f) << 16) |
            ((int)(255 * frgb[1] + .5f) << 8) |
            (int)(255 * frgb[2] + .5f);
    }
    
    public static int getRGB(double[] frgb) {
        return 0xFF000000 |
            ((int)(255 * frgb[0] + .5) << 16) |
            ((int)(255 * frgb[1] + .5) << 8) |
            (int)(255 * frgb[2] + .5);
    }
}
