/* Copyright (c) 2010 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.util;

/**
 * Utility class for Base36.
 * 
 * @author Jesper Öqvist (jesper@llbit.se)
 */
public class Base36 {

	/**
	 * Encode an integer to Base36 form.
	 * 
	 * @param x
	 * @return
	 */
	public static String encode(int x) {
		String result = "";
		int t = Math.abs(x);
		do {
			int p = t % 36;
			if (p < 10)
				result = ""+p+result;
			else
				result = ""+(char)('a'+(p-10))+result;
			
			t /= 36;
		} while (t > 0);
		if (x < 0)
			result = "-"+result;
		return result;
	}
	
	/**
	 * Decode a Base36 number.
	 * 
	 * @param x
	 * @return
	 */
	public static int decode(String x) {
		byte[] bytes = x.toLowerCase().getBytes();
		int result = 0;
		boolean signed = false;
		if (bytes.length > 0 && bytes[0] == '-')
			signed = true;
		for (int i = signed ? 1 : 0; i < bytes.length; ++i) {
			result *= 36;
			int b = bytes[i] & 0xFF;
			if (b >= 'a' && b <= 'z') {
				result += 10 + b - 'a';
			} else if (b >= '0' && b <= '9') {
				result += b - '0';
			}
		}
		return signed ? -result : result;
	}

	public static boolean validate(String x) {
		byte[] bytes = x.toLowerCase().getBytes();
		boolean signed = false;
		if (bytes.length > 0 && bytes[0] == '-')
			signed = true;
		if (signed && bytes.length == 1 || (!signed && bytes.length == 0))
			return false;
		for (int i = signed ? 1 : 0; i < bytes.length; ++i) {
			int b = bytes[i] & 0xFF;
			if (!(b >= 'a' && b <= 'z') &&
					!(b >= '0' && b <= '9'))
				return false;
		}
		return true;
	}

}
